@@ -57,7 +57,6 @@
 static bool qIsNaNPoint(const QPointF &p) {
     return qIsNaN(p.x()) || qIsNaN(p.y());
 }
-static const qreal DefaultMarkerWidth = 3.0;
 
 KoPathShapePrivate::KoPathShapePrivate(KoPathShape *q)
     : KoTosContainerPrivate(q),
@@ -164,8 +163,8 @@ void KoPathShape::saveContourOdf(KoShapeSavingContext &context, const QSizeF &sc
 
         if (currPoint && !(currPoint->activeControlPoint1() || currPoint->activeControlPoint2())) {
             context.xmlWriter().startElement("draw:contour-polygon");
-            context.xmlWriter().addAttributePt("svg:width", size().width());
-            context.xmlWriter().addAttributePt("svg:height", size().height());
+            context.xmlWriter().addAttribute("svg:width", size().width());
+            context.xmlWriter().addAttribute("svg:height", size().height());
 
             const QSizeF s(size());
             QString viewBox = QString("0 0 %1 %2").arg(qRound(1000*s.width())).arg(qRound(1000*s.height()));
@@ -601,7 +600,7 @@ QRectF KoPathShape::boundingRect() const
 QSizeF KoPathShape::size() const
 {
     // don't call boundingRect here as it uses absoluteTransformation
-    // which itself uses size() -> leads to infinite reccursion
+    // which itself uses size() -> leads to infinite recursion
     return outlineRect().size();
 }
 
@@ -808,12 +807,14 @@ QPointF KoPathShape::normalize()
 
 void KoPathShapePrivate::map(const QTransform &matrix)
 {
-    Q_Q(KoPathShape);
     KoSubpathList::const_iterator pathIt(subpaths.constBegin());
     for (; pathIt != subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
-            (*it)->map(matrix);
+            // It's possible there are null points in the map...
+            if (*it) {
+                (*it)->map(matrix);
+            }
         }
     }
 }
@@ -1386,7 +1387,6 @@ void KoPathShapePrivate::closeMergeSubpath(KoSubpath *subpath)
 
 KoSubpath *KoPathShapePrivate::subPath(int subpathIndex) const
 {
-    Q_Q(const KoPathShape);
     if (subpathIndex < 0 || subpathIndex >= subpaths.size())
         return 0;
 
@@ -1492,7 +1492,6 @@ char nodeType(const KoPathPoint * point)
 
 QString KoPathShapePrivate::nodeTypes() const
 {
-    Q_Q(const KoPathShape);
     QString types;
     KoSubpathList::const_iterator pathIt(subpaths.constBegin());
     for (; pathIt != subpaths.constEnd(); ++pathIt) {
@@ -1527,7 +1526,6 @@ void updateNodeType(KoPathPoint * point, const QChar & nodeType)
 
 void KoPathShapePrivate::loadNodeTypes(const KoXmlElement &element)
 {
-    Q_Q(KoPathShape);
     if (element.hasAttributeNS(KoXmlNS::calligra, "nodeTypes")) {
         QString nodeTypes = element.attributeNS(KoXmlNS::calligra, "nodeTypes");
         QString::const_iterator nIt(nodeTypes.constBegin());
