@@ -209,6 +209,10 @@ void KisEdgeDetectionKernel::applyEdgeDetection(KisPaintDeviceSP device,
         KisPaintDeviceSP x_denormalised = new KisPaintDevice(device->colorSpace());
         KisPaintDeviceSP y_denormalised = new KisPaintDevice(device->colorSpace());
 
+        x_denormalised->prepareClone(device);
+        y_denormalised->prepareClone(device);
+
+
         KisConvolutionKernelSP kernelHorizLeftRight = KisEdgeDetectionKernel::createHorizontalKernel(xRadius, type);
         KisConvolutionKernelSP kernelVerticalTopBottom = KisEdgeDetectionKernel::createVerticalKernel(yRadius, type);
 
@@ -237,6 +241,9 @@ void KisEdgeDetectionKernel::applyEdgeDetection(KisPaintDeviceSP device,
         KisSequentialIterator finalIt(device, rect);
         const int pixelSize = device->colorSpace()->pixelSize();
         const int channels = device->colorSpace()->channelCount();
+        const int alphaPos = device->colorSpace()->alphaPos();
+        KIS_SAFE_ASSERT_RECOVER_RETURN(alphaPos >= 0);
+
         QVector<float> yNormalised(channels);
         QVector<float> xNormalised(channels);
         QVector<float> finalNorm(channels);
@@ -269,6 +276,7 @@ void KisEdgeDetectionKernel::applyEdgeDetection(KisPaintDeviceSP device,
                 memcpy(finalIt.rawData(), col.data(), pixelSize);
             } else {
                 quint8* f = finalIt.rawData();
+                finalNorm[alphaPos] = 1.0;
                 device->colorSpace()->fromNormalisedChannelsValue(f, finalNorm);
                 memcpy(finalIt.rawData(), f, pixelSize);
             }
@@ -294,6 +302,8 @@ void KisEdgeDetectionKernel::applyEdgeDetection(KisPaintDeviceSP device,
 
         if (writeToAlpha) {
             KisPaintDeviceSP denormalised = new KisPaintDevice(device->colorSpace());
+            denormalised->prepareClone(device);
+
             KisConvolutionPainter kernelP(denormalised);
             kernelP.setChannelFlags(channelFlags);
             kernelP.setProgress(progressUpdater);
@@ -327,6 +337,13 @@ void KisEdgeDetectionKernel::applyEdgeDetection(KisPaintDeviceSP device,
                                 srcTopLeft - QPoint(0, ceil(center)),
                                 srcTopLeft - QPoint(0, ceil(center)),
                                 rect.size() + QSize(0, 2 * ceil(center)), BORDER_REPEAT);
+
+            KisSequentialIterator finalIt(device, rect);
+            int numConseqPixels = finalIt.nConseqPixels();
+            while (finalIt.nextPixels(numConseqPixels)) {
+                numConseqPixels = finalIt.nConseqPixels();
+                device->colorSpace()->setOpacity(finalIt.rawData(), 1.0, numConseqPixels);
+            }
         }
     }
 }
@@ -348,6 +365,8 @@ void KisEdgeDetectionKernel::convertToNormalMap(KisPaintDeviceSP device,
     finalPainter.setProgress(progressUpdater);
     KisPaintDeviceSP x_denormalised = new KisPaintDevice(device->colorSpace());
     KisPaintDeviceSP y_denormalised = new KisPaintDevice(device->colorSpace());
+    x_denormalised->prepareClone(device);
+    y_denormalised->prepareClone(device);
 
     KisConvolutionKernelSP kernelHorizLeftRight = KisEdgeDetectionKernel::createHorizontalKernel(yRadius, type, true, !channelFlip[1]);
     KisConvolutionKernelSP kernelVerticalTopBottom = KisEdgeDetectionKernel::createVerticalKernel(xRadius, type, true, !channelFlip[0]);
@@ -359,9 +378,9 @@ void KisEdgeDetectionKernel::convertToNormalMap(KisPaintDeviceSP device,
     horizPainterLR.setChannelFlags(channelFlags);
     horizPainterLR.setProgress(progressUpdater);
     horizPainterLR.applyMatrix(kernelHorizLeftRight, device,
-                               srcTopLeft - QPoint(0, ceil(horizontalCenter)),
-                               srcTopLeft - QPoint(0, ceil(horizontalCenter)),
-                               rect.size() + QSize(0, 2 * ceil(horizontalCenter)), BORDER_REPEAT);
+                               srcTopLeft - QPoint(ceil(horizontalCenter), 0),
+                               srcTopLeft - QPoint(ceil(horizontalCenter), 0),
+                               rect.size() + QSize(2 * ceil(horizontalCenter), 0), BORDER_REPEAT);
 
 
     KisConvolutionPainter verticalPainterTB(x_denormalised);
@@ -377,6 +396,9 @@ void KisEdgeDetectionKernel::convertToNormalMap(KisPaintDeviceSP device,
     KisSequentialIterator finalIt(device, rect);
     const int pixelSize = device->colorSpace()->pixelSize();
     const int channels = device->colorSpace()->channelCount();
+    const int alphaPos = device->colorSpace()->alphaPos();
+    KIS_SAFE_ASSERT_RECOVER_RETURN(alphaPos >= 0);
+
     QVector<float> yNormalised(channels);
     QVector<float> xNormalised(channels);
     QVector<float> finalNorm(channels);
@@ -396,6 +418,8 @@ void KisEdgeDetectionKernel::convertToNormalMap(KisPaintDeviceSP device,
             finalNorm[device->colorSpace()->channels().at(channelOrder[c])->displayPosition()] = (normal[channelOrder[c]]/2)+0.5;
         }
 
+        finalNorm[alphaPos]= 1.0;
+
         quint8* pixel = finalIt.rawData();
         device->colorSpace()->fromNormalisedChannelsValue(pixel, finalNorm);
         memcpy(finalIt.rawData(), pixel, pixelSize);
