@@ -155,6 +155,7 @@ public:
 
     KisSignalCompressor regionOfInterestUpdateCompressor;
     QRect regionOfInterest;
+    qreal regionOfInterestMargin = 0.25;
 
     QRect renderingLimit;
     int isBatchUpdateActive = 0;
@@ -222,6 +223,7 @@ void KisCanvas2::setup()
     KisConfig cfg(true);
     m_d->vastScrolling = cfg.vastScrolling();
     m_d->lodAllowedInImage = cfg.levelOfDetailEnabled();
+    m_d->regionOfInterestMargin = KisImageConfig(true).animationCacheRegionOfInterestMargin();
 
     createCanvas(cfg.useOpenGL());
 
@@ -700,26 +702,12 @@ void KisCanvas2::setProofingOptions(bool softProof, bool gamutCheck)
         m_d->proofingConfig = cfg.defaultProofingconfiguration();
     }
     KoColorConversionTransformation::ConversionFlags conversionFlags = m_d->proofingConfig->conversionFlags;
-#if QT_VERSION >= 0x050700
-
     if (this->image()->colorSpace()->colorDepthId().id().contains("U")) {
         conversionFlags.setFlag(KoColorConversionTransformation::SoftProofing, softProof);
         if (softProof) {
             conversionFlags.setFlag(KoColorConversionTransformation::GamutCheck, gamutCheck);
         }
     }
-#else
-    if (this->image()->colorSpace()->colorDepthId().id().contains("U")) {
-        conversionFlags |= KoColorConversionTransformation::SoftProofing;
-    } else {
-        conversionFlags = conversionFlags & ~KoColorConversionTransformation::SoftProofing;
-    }
-    if (gamutCheck && softProof && this->image()->colorSpace()->colorDepthId().id().contains("U")) {
-        conversionFlags |= KoColorConversionTransformation::GamutCheck;
-    } else {
-        conversionFlags = conversionFlags & ~KoColorConversionTransformation::GamutCheck;
-    }
-#endif
     m_d->proofingConfig->conversionFlags = conversionFlags;
 
     m_d->proofingConfigUpdated = true;
@@ -973,12 +961,12 @@ void KisCanvas2::slotUpdateRegionOfInterest()
 {
     const QRect oldRegionOfInterest = m_d->regionOfInterest;
 
-    const qreal ratio = 0.25;
+    const qreal ratio = m_d->regionOfInterestMargin;
     const QRect proposedRoi = KisAlgebra2D::blowRect(m_d->coordinatesConverter->widgetRectInImagePixels(), ratio).toAlignedRect();
 
     const QRect imageRect = m_d->coordinatesConverter->imageRectInImagePixels();
 
-    m_d->regionOfInterest = imageRect.contains(proposedRoi) ? proposedRoi : imageRect;
+    m_d->regionOfInterest = proposedRoi & imageRect;
 
     if (m_d->regionOfInterest != oldRegionOfInterest) {
         emit sigRegionOfInterestChanged(m_d->regionOfInterest);
@@ -1087,6 +1075,7 @@ void KisCanvas2::slotConfigChanged()
 {
     KisConfig cfg(true);
     m_d->vastScrolling = cfg.vastScrolling();
+    m_d->regionOfInterestMargin = KisImageConfig(true).animationCacheRegionOfInterestMargin();
 
     resetCanvas(cfg.useOpenGL());
 
