@@ -32,6 +32,7 @@
 #include "kis_image.h"
 #include "kis_config.h"
 #include "KisPart.h"
+#include "KisOpenGLModeProber.h"
 
 #ifdef HAVE_OPENEXR
 #include <half.h>
@@ -59,17 +60,13 @@ KisOpenGLImageTextures::ImageTexturesMap KisOpenGLImageTextures::imageTexturesMa
 KisOpenGLImageTextures::KisOpenGLImageTextures()
     : m_image(0)
     , m_monitorProfile(0)
-    , m_proofingConfig(0)
-    , m_createNewProofingTransform(true)
-    , m_tilesDestinationColorSpace(0)
     , m_internalColorManagementActive(true)
     , m_checkerTexture(0)
     , m_glFuncs(0)
-    , m_allChannelsSelected(true)
     , m_useOcio(false)
     , m_initialized(false)
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     m_renderingIntent = (KoColorConversionTransformation::Intent)cfg.monitorRenderIntent();
 
     m_conversionFlags = KoColorConversionTransformation::HighQuality;
@@ -86,12 +83,9 @@ KisOpenGLImageTextures::KisOpenGLImageTextures(KisImageWSP image,
     , m_monitorProfile(monitorProfile)
     , m_renderingIntent(renderingIntent)
     , m_conversionFlags(conversionFlags)
-    , m_createNewProofingTransform(true)
-    , m_tilesDestinationColorSpace(0)
     , m_internalColorManagementActive(true)
     , m_checkerTexture(0)
     , m_glFuncs(0)
-    , m_allChannelsSelected(true)
     , m_useOcio(false)
     , m_initialized(false)
 {
@@ -111,13 +105,13 @@ void KisOpenGLImageTextures::initGL(QOpenGLFunctions *f)
     // we use local static object for creating pools shared among
     // different images
     static KisTextureTileInfoPoolRegistry s_poolRegistry;
-    m_infoChunksPool = s_poolRegistry.getPool(m_texturesInfo.width, m_texturesInfo.height);
+    m_updateInfoBuilder.setTextureInfoPool(s_poolRegistry.getPool(m_texturesInfo.width, m_texturesInfo.height));
 
     m_glFuncs->glGenTextures(1, &m_checkerTexture);
-    createImageTextureTiles();
+    recreateImageTextureTiles();
 
     KisOpenGLUpdateInfoSP info = updateCache(m_image->bounds(), m_image);
-    recalculateCache(info);
+    recalculateCache(info, false);
 }
 
 KisOpenGLImageTextures::~KisOpenGLImageTextures()
@@ -142,7 +136,7 @@ KisImageSP KisOpenGLImageTextures::image() const
 
 bool KisOpenGLImageTextures::imageCanShareTextures()
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     if (cfg.useOcio()) return false;
     if (KisPart::instance()->mainwindowCount() == 1) return true;
     if (qApp->desktop()->screenCount() == 1) return true;
@@ -180,22 +174,16 @@ KisOpenGLImageTexturesSP KisOpenGLImageTextures::getImageTextures(KisImageWSP im
     }
 }
 
-QRect KisOpenGLImageTextures::calculateTileRect(int col, int row) const
-{
-    return m_image->bounds() &
-            QRect(col * m_texturesInfo.effectiveWidth,
-                  row * m_texturesInfo.effectiveHeight,
-                  m_texturesInfo.effectiveWidth,
-                  m_texturesInfo.effectiveHeight);
-}
-
-void KisOpenGLImageTextures::createImageTextureTiles()
+void KisOpenGLImageTextures::recreateImageTextureTiles()
 {
 
     destroyImageTextureTiles();
     updateTextureFormat();
 
-    if (!m_tilesDestinationColorSpace) {
+    const KoColorSpace *tilesDestinationColorSpace =
+        m_updateInfoBuilder.destinationColorSpace();
+
+    if (!tilesDestinationColorSpace) {
         qDebug() << "No destination colorspace!!!!";
         return;
     }
@@ -208,10 +196,10 @@ void KisOpenGLImageTextures::createImageTextureTiles()
     m_numCols = lastCol + 1;
 
     // Default color is transparent black
-    const int pixelSize = m_tilesDestinationColorSpace->pixelSize();
+    const int pixelSize = tilesDestinationColorSpace->pixelSize();
     QByteArray emptyTileData((m_texturesInfo.width) * (m_texturesInfo.height) * pixelSize, 0);
 
-    KisConfig config;
+    KisConfig config(true);
     KisOpenGL::FilterMode mode = (KisOpenGL::FilterMode)config.openGLFilteringMode();
 
     QOpenGLContext *ctx = QOpenGLContext::currentContext();
@@ -224,7 +212,7 @@ void KisOpenGLImageTextures::createImageTextureTiles()
         m_textureTiles.reserve((lastRow+1)*m_numCols);
         for (int row = 0; row <= lastRow; row++) {
             for (int col = 0; col <= lastCol; col++) {
-                QRect tileRect = calculateTileRect(col, row);
+                QRect tileRect = m_updateInfoBuilder.calculateEffectiveTileRect(col, row, m_image->bounds());
 
                 KisTextureTile *tile = new KisTextureTile(tileRect,
                                                           &m_texturesInfo,
@@ -266,111 +254,11 @@ KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCacheNoConversion(const QRec
 // TODO: add sanity checks about the conformance of the passed srcImage!
 KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCacheImpl(const QRect& rect, KisImageSP srcImage, bool convertColorSpace)
 {
-    const KoColorSpace *dstCS = m_tilesDestinationColorSpace;
-
-    ConversionOptions options;
-
-    if (convertColorSpace) {
-        options = ConversionOptions(dstCS, m_renderingIntent, m_conversionFlags);
-    }
-
-    KisOpenGLUpdateInfoSP info = new KisOpenGLUpdateInfo(options);
-
-    QRect updateRect = rect & srcImage->bounds();
-    if (updateRect.isEmpty() || !(m_initialized)) return info;
-
-    /**
-     * Why the rect is artificial? That's easy!
-     * It does not represent any real piece of the image. It is
-     * intentionally stretched to get through the overlappping
-     * stripes of neutrality and poke neighbouring tiles.
-     * Thanks to the rect we get the coordinates of all the tiles
-     * involved into update process
-     */
-
-    QRect artificialRect = stretchRect(updateRect, m_texturesInfo.border);
-    artificialRect &= srcImage->bounds();
-
-    int firstColumn = xToCol(artificialRect.left());
-    int lastColumn = xToCol(artificialRect.right());
-    int firstRow = yToRow(artificialRect.top());
-    int lastRow = yToRow(artificialRect.bottom());
-
-    QBitArray channelFlags; // empty by default
-
-    if (m_channelFlags.size() != srcImage->projection()->colorSpace()->channels().size()) {
-        setChannelFlags(QBitArray());
-    }
-    if (!m_useOcio) { // Ocio does its own channel flipping
-        if (!m_allChannelsSelected) { // and we do it only if necessary
-            channelFlags = m_channelFlags;
-        }
-    }
-
-    qint32 numItems = (lastColumn - firstColumn + 1) * (lastRow - firstRow + 1);
-    info->tileList.reserve(numItems);
-
-    const QRect bounds = srcImage->bounds();
-    const int levelOfDetail = srcImage->currentLevelOfDetail();
-
-    QRect alignedUpdateRect = updateRect;
-    QRect alignedBounds = bounds;
-
-    if (levelOfDetail) {
-        alignedUpdateRect = KisLodTransform::alignedRect(alignedUpdateRect, levelOfDetail);
-        alignedBounds = KisLodTransform::alignedRect(alignedBounds, levelOfDetail);
-    }
-
-    for (int col = firstColumn; col <= lastColumn; col++) {
-        for (int row = firstRow; row <= lastRow; row++) {
-
-            const QRect tileRect = calculateTileRect(col, row);
-            const QRect tileTextureRect = stretchRect(tileRect, m_texturesInfo.border);
-
-            QRect alignedTileTextureRect = levelOfDetail ?
-                        KisLodTransform::alignedRect(tileTextureRect, levelOfDetail) :
-                        tileTextureRect;
-
-            KisTextureTileUpdateInfoSP tileInfo(
-                        new KisTextureTileUpdateInfo(col, row,
-                                                     alignedTileTextureRect,
-                                                     alignedUpdateRect,
-                                                     alignedBounds,
-                                                     levelOfDetail,
-                                                     m_infoChunksPool));
-            // Don't update empty tiles
-            if (tileInfo->valid()) {
-                tileInfo->retrieveData(srcImage->projection(), channelFlags, m_onlyOneChannelSelected, m_selectedChannelIndex);
-
-                //create transform
-                if (m_createNewProofingTransform) {
-                    const KoColorSpace *proofingSpace = KoColorSpaceRegistry::instance()->colorSpace(m_proofingConfig->proofingModel,m_proofingConfig->proofingDepth,m_proofingConfig->proofingProfile);
-                    m_proofingTransform.reset(tileInfo->generateProofingTransform(dstCS, proofingSpace, m_renderingIntent, m_proofingConfig->intent, m_proofingConfig->conversionFlags, m_proofingConfig->warningColor, m_proofingConfig->adaptationState));
-                    m_createNewProofingTransform = false;
-                }
-
-                if (convertColorSpace) {
-                    if (m_proofingConfig && m_proofingTransform && m_proofingConfig->conversionFlags.testFlag(KoColorConversionTransformation::SoftProofing)) {
-                        tileInfo->proofTo(dstCS, m_proofingConfig->conversionFlags, m_proofingTransform.data());
-                    } else {
-                        tileInfo->convertTo(dstCS, m_renderingIntent, m_conversionFlags);
-                    }
-                }
-
-                info->tileList.append(tileInfo);
-            }
-            else {
-                dbgUI << "Trying to create an empty tileinfo record" << col << row << tileTextureRect << updateRect << srcImage->bounds();
-            }
-        }
-    }
-
-    info->assignDirtyImageRect(rect);
-    info->assignLevelOfDetail(levelOfDetail);
-    return info;
+    if (!m_initialized) return new KisOpenGLUpdateInfo();
+    return m_updateInfoBuilder.buildUpdateInfo(rect, srcImage, convertColorSpace);
 }
 
-void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info)
+void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info, bool blockMipmapRegeneration)
 {
     if (!m_initialized) {
         dbgUI << "OpenGL: Tried to edit image texture cache before it was initialized.";
@@ -385,7 +273,7 @@ void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info)
         KisTextureTile *tile = getTextureTileCR(tileInfo->tileCol(), tileInfo->tileRow());
         KIS_ASSERT_RECOVER_RETURN(tile);
 
-        tile->update(*tileInfo);
+        tile->update(*tileInfo, blockMipmapRegeneration);
     }
 }
 
@@ -453,7 +341,12 @@ void KisOpenGLImageTextures::updateConfig(bool useBuffer, int NumMipmapLevels)
 
 void KisOpenGLImageTextures::slotImageSizeChanged(qint32 /*w*/, qint32 /*h*/)
 {
-    createImageTextureTiles();
+    recreateImageTextureTiles();
+}
+
+KisOpenGLUpdateInfoBuilder &KisOpenGLImageTextures::updateInfoBuilder()
+{
+    return m_updateInfoBuilder;
 }
 
 void KisOpenGLImageTextures::setMonitorProfile(const KoColorProfile *monitorProfile, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags)
@@ -463,39 +356,56 @@ void KisOpenGLImageTextures::setMonitorProfile(const KoColorProfile *monitorProf
     m_renderingIntent = renderingIntent;
     m_conversionFlags = conversionFlags;
 
-    createImageTextureTiles();
+    recreateImageTextureTiles();
+}
+
+bool KisOpenGLImageTextures::setImageColorSpace(const KoColorSpace *cs)
+{
+    Q_UNUSED(cs);
+    // TODO: implement lazy update: do not re-upload textures when not needed
+
+    recreateImageTextureTiles();
+    return true;
 }
 
 void KisOpenGLImageTextures::setChannelFlags(const QBitArray &channelFlags)
 {
-    m_channelFlags = channelFlags;
+    QBitArray targetChannelFlags = channelFlags;
     int selectedChannels = 0;
     const KoColorSpace *projectionCs = m_image->projection()->colorSpace();
     QList<KoChannelInfo*> channelInfo = projectionCs->channels();
 
-    if (m_channelFlags.size() != channelInfo.size()) {
-        m_channelFlags = QBitArray();
+    if (targetChannelFlags.size() != channelInfo.size()) {
+        targetChannelFlags = QBitArray();
     }
 
-    for (int i = 0; i < m_channelFlags.size(); ++i) {
-        if (m_channelFlags.testBit(i) && channelInfo[i]->channelType() == KoChannelInfo::COLOR) {
+    int selectedChannelIndex = -1;
+
+    for (int i = 0; i < targetChannelFlags.size(); ++i) {
+        if (targetChannelFlags.testBit(i) && channelInfo[i]->channelType() == KoChannelInfo::COLOR) {
             selectedChannels++;
-            m_selectedChannelIndex = i;
+            selectedChannelIndex = i;
         }
     }
-    m_allChannelsSelected = (selectedChannels == m_channelFlags.size());
-    m_onlyOneChannelSelected = (selectedChannels == 1);
+    const bool allChannelsSelected = (selectedChannels == targetChannelFlags.size());
+    const bool onlyOneChannelSelected = (selectedChannels == 1);
+
+    // OCIO has its own channel swizzling
+    if (allChannelsSelected || m_useOcio) {
+        m_updateInfoBuilder.setChannelFlags(QBitArray(), false, -1);
+    } else {
+        m_updateInfoBuilder.setChannelFlags(targetChannelFlags, onlyOneChannelSelected, selectedChannelIndex);
+    }
 }
 
 void KisOpenGLImageTextures::setProofingConfig(KisProofingConfigurationSP proofingConfig)
 {
-    m_proofingConfig = proofingConfig;
-    m_createNewProofingTransform = true;
+    m_updateInfoBuilder.setProofingConfig(proofingConfig);
 }
 
 void KisOpenGLImageTextures::getTextureSize(KisGLTexturesInfo *texturesInfo)
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
 
     const GLint preferredTextureSize = cfg.openGLTextureSize();
 
@@ -515,6 +425,10 @@ void KisOpenGLImageTextures::getTextureSize(KisGLTexturesInfo *texturesInfo)
 
     texturesInfo->effectiveWidth = texturesInfo->width - 2 * texturesInfo->border;
     texturesInfo->effectiveHeight = texturesInfo->height - 2 * texturesInfo->border;
+
+    m_updateInfoBuilder.setTextureBorder(texturesInfo->border);
+    m_updateInfoBuilder.setEffectiveTextureSize(
+        QSize(texturesInfo->effectiveWidth, texturesInfo->effectiveHeight));
 }
 
 bool KisOpenGLImageTextures::internalColorManagementActive() const
@@ -528,7 +442,7 @@ bool KisOpenGLImageTextures::setInternalColorManagementActive(bool value)
 
     if (needsFinalRegeneration) {
         m_internalColorManagementActive = value;
-        createImageTextureTiles();
+        recreateImageTextureTiles();
 
         // at this point the value of m_internalColorManagementActive might
         // have been forcely reverted to 'false' in case of some problems
@@ -537,6 +451,43 @@ bool KisOpenGLImageTextures::setInternalColorManagementActive(bool value)
     return needsFinalRegeneration;
 }
 
+namespace {
+void initializeRGBA16FTextures(QOpenGLContext *ctx, KisGLTexturesInfo &texturesInfo, KoID &destinationColorDepthId)
+{
+    if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
+        texturesInfo.internalFormat = GL_RGBA16F;
+        dbgUI << "Using half (GLES or GL3)";
+    } else if (ctx->hasExtension("GL_ARB_texture_float")) {
+        texturesInfo.internalFormat = GL_RGBA16F_ARB;
+        dbgUI << "Using ARB half";
+    }
+    else if (ctx->hasExtension("GL_ATI_texture_float")) {
+        texturesInfo.internalFormat = GL_RGBA_FLOAT16_ATI;
+        dbgUI << "Using ATI half";
+    }
+
+    bool haveBuiltInOpenExr = false;
+#ifdef HAVE_OPENEXR
+    haveBuiltInOpenExr = true;
+#endif
+
+    if (haveBuiltInOpenExr && (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3())) {
+        texturesInfo.type = GL_HALF_FLOAT;
+        destinationColorDepthId = Float16BitsColorDepthID;
+        dbgUI << "Pixel type half (GLES or GL3)";
+    } else if (haveBuiltInOpenExr && ctx->hasExtension("GL_ARB_half_float_pixel")) {
+        texturesInfo.type = GL_HALF_FLOAT_ARB;
+        destinationColorDepthId = Float16BitsColorDepthID;
+        dbgUI << "Pixel type half";
+    } else {
+        texturesInfo.type = GL_FLOAT;
+        destinationColorDepthId = Float32BitsColorDepthID;
+        dbgUI << "Pixel type float";
+    }
+    texturesInfo.format = GL_RGBA;
+}
+}
+
 void KisOpenGLImageTextures::updateTextureFormat()
 {
     QOpenGLContext *ctx = QOpenGLContext::currentContext();
@@ -559,8 +510,9 @@ void KisOpenGLImageTextures::updateTextureFormat()
         }
     }
 
-    KoID colorModelId = m_image->colorSpace()->colorModelId();
-    KoID colorDepthId = m_image->colorSpace()->colorDepthId();
+    const bool useHDRMode = KisOpenGLModeProber::instance()->useHDRMode();
+    const KoID colorModelId = m_image->colorSpace()->colorModelId();
+    const KoID colorDepthId = useHDRMode ? Float16BitsColorDepthID : m_image->colorSpace()->colorDepthId();
 
     KoID destinationColorModelId = RGBAColorModelID;
     KoID destinationColorDepthId = Integer8BitsColorDepthID;
@@ -569,38 +521,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
 
     if (colorModelId == RGBAColorModelID) {
         if (colorDepthId == Float16BitsColorDepthID) {
-
-            if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
-                m_texturesInfo.internalFormat = GL_RGBA16F;
-                dbgUI << "Using half (GLES or GL3)";
-            } else if (ctx->hasExtension("GL_ARB_texture_float")) {
-                m_texturesInfo.internalFormat = GL_RGBA16F_ARB;
-                dbgUI << "Using ARB half";
-            }
-            else if (ctx->hasExtension("GL_ATI_texture_float")) {
-                m_texturesInfo.internalFormat = GL_RGBA_FLOAT16_ATI;
-                dbgUI << "Using ATI half";
-            }
-
-            bool haveBuiltInOpenExr = false;
-#ifdef HAVE_OPENEXR
-            haveBuiltInOpenExr = true;
-#endif
-
-            if (haveBuiltInOpenExr && (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3())) {
-                m_texturesInfo.type = GL_HALF_FLOAT;
-                destinationColorDepthId = Float16BitsColorDepthID;
-                dbgUI << "Pixel type half (GLES or GL3)";
-            } else if (haveBuiltInOpenExr && ctx->hasExtension("GL_ARB_half_float_pixel")) {
-                m_texturesInfo.type = GL_HALF_FLOAT_ARB;
-                destinationColorDepthId = Float16BitsColorDepthID;
-                dbgUI << "Pixel type half";
-            } else {
-                m_texturesInfo.type = GL_FLOAT;
-                destinationColorDepthId = Float32BitsColorDepthID;
-                dbgUI << "Pixel type float";
-            }
-            m_texturesInfo.format = GL_RGBA;
+            initializeRGBA16FTextures(ctx, m_texturesInfo, destinationColorDepthId);
         }
         else if (colorDepthId == Float32BitsColorDepthID) {
             if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
@@ -637,6 +558,9 @@ void KisOpenGLImageTextures::updateTextureFormat()
             m_texturesInfo.format = GL_BGRA;
             destinationColorDepthId = Integer16BitsColorDepthID;
             dbgUI << "Using conversion to 16 bits rgba";
+        } else if (colorDepthId == Float16BitsColorDepthID && KisOpenGL::hasOpenGLES()) {
+            // TODO: try removing opengl es limit
+            initializeRGBA16FTextures(ctx, m_texturesInfo, destinationColorDepthId);
         }
         // TODO: for ANGLE, see if we can convert to 16f to support 10-bit display
     }
@@ -644,16 +568,15 @@ void KisOpenGLImageTextures::updateTextureFormat()
     if (!m_internalColorManagementActive &&
             colorModelId != destinationColorModelId) {
 
-        KisConfig cfg;
+        KisConfig cfg(false);
         KisConfig::OcioColorManagementMode cm = cfg.ocioColorManagementMode();
 
         if (cm != KisConfig::INTERNAL) {
-            QMessageBox::critical(0,
-                                  i18nc("@title:window", "Krita"),
-                                  i18n("You enabled OpenColorIO based color management, but your image is not an RGB image.\n"
-                                       "OpenColorIO-based color management only works with RGB images.\n"
-                                       "Please check the settings in the LUT docker.\n"
-                                       "OpenColorIO will now be deactivated."));
+            emit sigShowFloatingMessage(
+                i18n("You enabled OpenColorIO based color management, but your image is not an RGB image.\n"
+                     "OpenColorIO-based color management only works with RGB images.\n"
+                     "Please check the settings in the LUT docker.\n"
+                     "OpenColorIO will now be deactivated."), 5000, true);
         }
 
         warnUI << "WARNING: Internal color management was forcibly enabled";
@@ -676,9 +599,14 @@ void KisOpenGLImageTextures::updateTextureFormat()
      *       would not be called when not needed (DK)
      */
 
-    m_tilesDestinationColorSpace =
+    const KoColorSpace *tilesDestinationColorSpace =
             KoColorSpaceRegistry::instance()->colorSpace(destinationColorModelId.id(),
                                                          destinationColorDepthId.id(),
                                                          profile);
+
+    m_updateInfoBuilder.setConversionOptions(
+        ConversionOptions(tilesDestinationColorSpace,
+                          m_renderingIntent,
+                          m_conversionFlags));
 }
 
